@@ -15,21 +15,21 @@ def test_stream():
     tmo.settings.set_thermo(['Water'], cache=True)
     stream = tmo.Stream(None, Water=1, T=300)
     assert [stream.chemicals.Water] == stream.available_chemicals
-    assert_allclose(stream.epsilon, 77.72395675564552)
-    assert_allclose(stream.alpha * 1e6, 0.1462889847474686)
-    assert_allclose(stream.nu, 8.596760626756933e-07)
-    assert_allclose(stream.Pr, 5.876560454361681)
+    assert_allclose(stream.epsilon, 77.70030000000003)
+    assert_allclose(stream.alpha * 1e6, 0.14330776454124503)
+    assert_allclose(stream.nu, 8.799123532986536e-07)
+    assert_allclose(stream.Pr, 6.14001869413997)
     assert_allclose(stream.Cn, 75.29555729396768)
     assert_allclose(stream.C, 75.29555729396768)
     assert_allclose(stream.Cp, 4.179538552493643)
     assert_allclose(stream.P_vapor, 3533.918074415897)
-    assert_allclose(stream.mu, 0.0008564676992124578)
-    assert_allclose(stream.kappa, 0.609138593165859)
-    assert_allclose(stream.rho, 996.2679390499142)
-    assert_allclose(stream.V, 1.808276598480142e-05)
-    assert_allclose(stream.H, 139.3139852692184)
-    assert_allclose(stream.S, 70.465818)
-    assert_allclose(stream.sigma, 0.07168596252716256)
+    assert_allclose(stream.mu, 0.0008766363688287887)
+    assert_allclose(stream.kappa, 0.5967303492959747)
+    assert_allclose(stream.rho, 996.2769195618362)
+    assert_allclose(stream.V, 1.80826029854462e-05)
+    assert_allclose(stream.H, 139.31398526921475)
+    assert_allclose(stream.S, 70.46581776376684)
+    assert_allclose(stream.sigma, 0.07176932405246211)
     assert_allclose(stream.z_mol, [1.0])
     assert_allclose(stream.z_mass, [1.0])
     assert_allclose(stream.z_vol, [1.0])
@@ -77,21 +77,21 @@ def test_multistream():
     tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True)
     stream = tmo.MultiStream(None, l=[('Water', 1)], T=300)
     assert [stream.chemicals.Water] == stream.available_chemicals
-    assert_allclose(stream.epsilon, 77.72395675564552)
-    assert_allclose(stream.alpha * 1e6, 0.1462889847474686)
-    assert_allclose(stream.nu, 8.596760626756933e-07)
-    assert_allclose(stream.Pr, 5.876560454361681)
+    assert_allclose(stream.epsilon, 77.70030000000003)
+    assert_allclose(stream.alpha * 1e6, 1.4330776454124502e-01)
+    assert_allclose(stream.nu, 8.799123532986536e-07)
+    assert_allclose(stream.Pr, 6.14001869413997)
     assert_allclose(stream.Cn, 75.29555729396768)
     assert_allclose(stream.C, 75.29555729396768)
     assert_allclose(stream.Cp, 4.179538552493643)
     assert_allclose(stream.P_vapor, 3533.918074415897)
-    assert_allclose(stream.mu, 0.0008564676992124578)
-    assert_allclose(stream.kappa, 0.609138593165859)
-    assert_allclose(stream.rho, 996.2679390499142)
-    assert_allclose(stream.V, 1.808276598480142e-05)
-    assert_allclose(stream.H, 139.3139852692184)
+    assert_allclose(stream.mu, 0.0008766363688287887)
+    assert_allclose(stream.kappa, 0.5967303492959747)
+    assert_allclose(stream.rho, 996.2769195618362)
+    assert_allclose(stream.V, 1.80826029854462e-05)
+    assert_allclose(stream.H, 139.31398526921475)
     assert_allclose(stream.S, 70.465818)
-    assert_allclose(stream.sigma, 0.07168596252716256)
+    assert_allclose(stream.sigma, 0.07176932405246211)
     assert_allclose(stream.z_mol, [1.0, 0.])
     assert_allclose(stream.z_mass, [1.0, 0.])
     assert_allclose(stream.z_vol, [1.0, 0.])
