@@ -26,7 +26,8 @@
 #     update_module(chemicals, numba)
 # use_numba_chemicals()
 # del use_numba_chemicals
-
+from . import thermo
+del thermo
 from . import (constants,
                base,
                utils,
@@ -79,4 +80,4 @@ pd.set_option('display.max_columns', 10)
 pd.set_option('max_colwidth', 35)
 del np, pd
 
-__version__ = "0.26.0"
+__version__ = "0.27.3"
