@@ -10,7 +10,7 @@
 from . import utils
 from .exceptions import UndefinedChemical
 from ._chemical import Chemical
-from .indexer import ChemicalIndexer, ChemicalGroupIndex, ChemicalSplitIndexer
+from .indexer import ChemicalIndexer, SplitIndexer
 import thermosteam as tmo
 import numpy as np
 
@@ -220,8 +220,8 @@ class Chemicals:
         >>> s = tmo.Stream('s', Substance=10)
         >>> s.rho
         Traceback (most recent call last):
-        DomainError: Substance (CAS: Substance) has no valid liquid molar 
-        volume model at T=298.15 K and P=101325 Pa
+        RuntimeError: No liquid molar volume method selected for 
+        component with CASRN 'Substance'
         
         """
         chemicals = tuple(self)
@@ -276,8 +276,6 @@ class CompiledChemicals(Chemicals):
         
     Attributes
     ----------
-    groups : dict[str, set[str]]
-        Groups of chemicals.
     tuple : tuple[Chemical]
         All compiled chemicals.
     size : int
@@ -360,9 +358,104 @@ class CompiledChemicals(Chemicals):
         """Do nothing, CompiledChemicals objects are already compiled.""" 
     
     def define_group(self, name, IDs):
+        """
+        Define a group of chemicals.
+        
+        Parameters
+        ----------
+        name : str
+            Name of group.
+        IDs : List[str]
+            IDs of chemicals in the group.
+            
+        Examples
+        --------
+        Create a chemical group and get the index:
+        
+        >>> import thermosteam as tmo
+        >>> chemicals = tmo.CompiledChemicals(['Water', 'Methanol', 'Ethanol'], cache=True)
+        >>> chemicals.define_group('Alcohol', ['Methanol', 'Ethanol'])
+        >>> chemicals.get_index('Alcohol')
+        [1, 2]
+        >>> chemicals.get_index(('Water', 'Alcohol'))
+        [0, [1, 2]]
+        
+        By defining a chemical group, you can conviniently use indexers
+        to retrieve the total value of the group:
+            
+        >>> # Single phase stream case
+        >>> tmo.settings.set_thermo(chemicals)
+        >>> s1 = tmo.Stream(ID='s1', Water=2)
+        >>> s1.imol['Alcohol']
+        0.
+        >>> s1.imol['Methanol', 'Ethanol'] = [2., 1.]
+        >>> s1.imol['Water', 'Alcohol']
+        array([2., 3.])
+        
+        >>> # Multi-phase stream case
+        >>> s2 = tmo.Stream(ID='s2', Water=2, Methanol=1, Ethanol=1)
+        >>> s2.vle(V=0.5, P=101325)
+        >>> s2.imol['Alcohol']
+        2.0
+        >>> s2.imol['l', 'Alcohol']
+        0.678
+        >>> s2.imol['l', ('Water', 'Alcohol')]
+        array([1.322, 0.678])
+        
+        Note that groups cannot be set using stream indexers as they represent 
+        the sum of chemicals in the group without a defined composition:
+            
+        >>> s1.imol['Alcohol'] = 3.
+        Traceback (most recent call last):
+        IndexError: 'Alcohol' is a chemical group; cannot set values by chemical group
+        
+        >>> s1.imol['Water', 'Alcohol'] = [3., 1.]
+        Traceback (most recent call last):
+        IndexError: 'Alcohol' is a chemical group; cannot set values by chemical group
+        
+        >>> s2.imol['Alcohol'] = 1.
+        Traceback (most recent call last):
+        IndexError: 'Alcohol' is a chemical group; cannot set values by chemical group
+        
+        For SplitIndexer objects, which reflect the separation of chemicals 
+        in two streams, getting and setting by chemical groups is OK because
+        group names correspond to nested indexes:
+            
+        >>> # Create a split-indexer
+        >>> indexer = chemicals.isplit(0.7)
+        >>> indexer.show()
+        SplitIndexer:
+         Water     0.7
+         Methanol  0.7
+         Ethanol   0.7
+        
+        >>> # Normal index
+        >>> indexer['Alcohol'] = [0.50, 0.80] # Methanol and ethanol
+        >>> indexer['Alcohol']
+        array([0.5, 0.8])
+        
+        >>> # Broadcasted index
+        >>> indexer['Alcohol'] = 0.9
+        >>> indexer['Alcohol']
+        array([0.9, 0.9])
+        
+        >>> # Nested index
+        >>> indexer['Water', 'Alcohol'] = [0.2, [0.6, 0.7]]
+        >>> indexer['Water', 'Alcohol']
+        array([0.2, array([0.6, 0.7])], dtype=object)
+        
+        >>> # Broadcasted and nested index
+        >>> indexer['Water', 'Alcohol'] = [0.2, 0.8]
+        >>> indexer['Water', 'Alcohol']
+        array([0.2, array([0.8, 0.8])], dtype=object)
+        
+        This feature allows splits to be easily defined for groups of chemicals 
+        in BioSTEAM.
+        
+        """
         IDs = tuple(IDs)
         index = utils.flattened(self.indices(IDs))
-        self._index[name] = ChemicalGroupIndex(index)
+        self._index[name] = index
         self.__dict__[name] = [self.tuple[i] for i in index]
     
     def refresh_constants(self):
@@ -418,7 +511,6 @@ class CompiledChemicals(Chemicals):
         size = len(IDs)
         index = tuple_(range(size))
         for i in chemicals: dct[i.CAS] = i
-        dct['groups'] = {}
         dct['tuple'] = chemicals
         dct['size'] = size
         dct['IDs'] = IDs
@@ -490,126 +582,11 @@ class CompiledChemicals(Chemicals):
         
         Examples
         --------
+        >>> from thermosteam import CompiledChemicals
         >>> chemicals = CompiledChemicals(['Water', 'Ethanol', 'Propane'], cache=True)
-        >>> chemicals.formula_array
-        array([[2., 6., 8.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 2., 3.],
-               [0., 0., 0.],
-               [1., 1., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.],
-               [0., 0., 0.]])
+        >>> chemicals.formula_array.sum(0)
+        array([ 3.,  9., 11.])
+        
         """
         try: return self._formula_array
         except: pass
@@ -805,19 +782,6 @@ class CompiledChemicals(Chemicals):
         >>> indexer['Ethanol']
         1.0
         
-        Index the sum of a chemical group:
-            
-        >>> chemicals.define_group('Alcohol', ['Methanol', 'Ethanol'])
-        >>> indexer['Methanol', 'Ethanol'] = [0.5, 2.0]
-        >>> indexer['Alcohol']
-        2.5
-        
-        Note that you cannot set values vy chemical group:
-        
-        >>> indexer['Alcohol'] = 2.0
-        Traceback (most recent call last):
-        IndexError: 'Alcohol' is a chemical group; cannot set values by chemical group
-        
         """
         array = self.array(IDs, data)
         return ChemicalIndexer.from_data(array, chemicals=self)
@@ -851,26 +815,13 @@ class CompiledChemicals(Chemicals):
         >>> indexer['Ethanol']
         1.0
         
-        Index the sum of a chemical group:
-            
-        >>> chemicals.define_group('Alcohol', ['Methanol', 'Ethanol'])
-        >>> indexer['Methanol', 'Ethanol'] = [0.5, 2.0]
-        >>> indexer['Alcohol']
-        2.5
-        
-        Note that you cannot set values vy chemical group:
-        
-        >>> indexer['Alcohol'] = 2.0
-        Traceback (most recent call last):
-        IndexError: 'Alcohol' is a chemical group; cannot set values by chemical group
-        
         """
         array = self.kwarray(ID_data)
         return ChemicalIndexer.from_data(array, chemicals=self)
 
     def isplit(self, split, order=None):
         """
-        Create a chemical-split indexer that represents chemical splits.
+        Create a SplitIndexer object that represents chemical splits.
     
         Parameters
         ----------   
@@ -889,7 +840,7 @@ class CompiledChemicals(Chemicals):
         >>> chemicals = CompiledChemicals(['Water', 'Methanol', 'Ethanol'], cache=True)
         >>> indexer = chemicals.isplit(dict(Water=0.5, Ethanol=1.))
         >>> indexer.show()
-        ChemicalSplitIndexer:
+        SplitIndexer:
          Water    0.5
          Ethanol  1
         
@@ -897,7 +848,7 @@ class CompiledChemicals(Chemicals):
         
         >>> indexer = chemicals.isplit([0.5, 1], ['Water', 'Ethanol'])
         >>> indexer.show()
-        ChemicalSplitIndexer:
+        SplitIndexer:
          Water    0.5
          Ethanol  1
            
@@ -905,7 +856,7 @@ class CompiledChemicals(Chemicals):
         
         >>> indexer = chemicals.isplit(0.75)
         >>> indexer.show()
-        ChemicalSplitIndexer:
+        SplitIndexer:
          Water     0.75
          Methanol  0.75
          Ethanol   0.75
@@ -914,30 +865,9 @@ class CompiledChemicals(Chemicals):
         
         >>> indexer = chemicals.isplit([0.5, 0, 1])
         >>> indexer.show()
-        ChemicalSplitIndexer:
+        SplitIndexer:
          Water    0.5
          Ethanol  1
-        
-        Define splits by group:
-        
-        >>> chemicals.define_group('Alcohol', ['Methanol', 'Ethanol']) 
-        >>> indexer['Alcohol'] = 0.8
-        >>> indexer.show()
-        ChemicalSplitIndexer:
-         Water     0.5
-         Methanol  0.8
-         Ethanol   0.8
-        
-        This is possible because ChemicalSplitIndexer objects can use nested
-        indices with broadcasting:
-        
-        >>> indexer['Alcohol'] = [0.70, 0.80]
-        >>> indexer['Alcohol']
-        array([0.7, 0.8])
-        
-        >>> indexer['Water', 'Alcohol'] = [0.2, [0.6, 0.7]]
-        >>> indexer['Water', 'Alcohol']
-        array([0.2, array([0.6, 0.7])], dtype=object)
 
         """
         if isinstance(split, dict):
@@ -945,14 +875,14 @@ class CompiledChemicals(Chemicals):
             order, split = zip(*split.items())
         
         if order:
-            isplit = ChemicalSplitIndexer.blank(chemicals=self)
+            isplit = SplitIndexer.blank(chemicals=self)
             isplit[tuple(order)] = split
         elif hasattr(split, '__len__'):
-            isplit = ChemicalSplitIndexer.from_data(np.asarray(split),
+            isplit = SplitIndexer.from_data(np.asarray(split),
                                                     chemicals=self)
         else:
             split = split * np.ones(self.size)
-            isplit = ChemicalSplitIndexer.from_data(split,
+            isplit = SplitIndexer.from_data(split,
                                                     chemicals=self)
         return isplit
 
@@ -1081,17 +1011,6 @@ class CompiledChemicals(Chemicals):
         TypeError: only strings, tuples, and ellipsis are valid index keys
 
         """
-        cache = self._index_cache
-        try: 
-            index = cache[IDs]
-        except KeyError: 
-            cache[IDs] = index = self._get_index(IDs)
-            utils.trim_cache(cache)
-        except TypeError:
-            raise TypeError("only strings, tuples, and ellipsis are valid index keys")
-        return index
-    
-    def _get_index(self, IDs):
         if isinstance(IDs, str):
             return self.index(IDs)
         elif isinstance(IDs, tuple):
@@ -1101,6 +1020,27 @@ class CompiledChemicals(Chemicals):
         else: # pragma: no cover
             raise TypeError("only strings, tuples, and ellipsis are valid index keys")    
     
+    def _get_index_and_kind(self, key):
+        index_cache = self._index_cache
+        if key in index_cache:
+            return index_cache[key]
+        else:
+            isa = isinstance
+            kind = 0 # [int] Kind of index: 0 - normal, 1 - chemical group, 2 - nested chemical group
+            if isa(key, str):
+                index = self.index(key)
+                if isa(index, list): kind = 1 
+            elif isa(key, tuple):
+                index = self.indices(key)
+                for i in index:
+                    if isa(i, list): kind = 2
+            elif key is ...:
+                index = slice(None)
+            else: # pragma: no cover
+                raise TypeError("only strings, tuples, and ellipsis are valid index keys")
+            index_cache[key] = index, kind
+        return index, kind
+    
     def __len__(self):
         return self.size
     
