@@ -108,7 +108,7 @@ class Stream:
     >>> s1.mass
     property_array([<Water: 20 kg/hr>, <Ethanol: 10 kg/hr>])
     >>> s1.vol
-    property_array([<Water: 0.02006 m^3/hr>, <Ethanol: 0.012722 m^3/hr>])
+    property_array([<Water: 0.02006 m^3/hr>, <Ethanol: 0.012724 m^3/hr>])
     
     These arrays work just like ordinary arrays, but the data is linked to the molar flows:
     
@@ -169,12 +169,12 @@ class Stream:
     Other thermodynamic properties are temperature and pressure dependent as well:
     
     >>> s1.rho # Density [kg/m3]
-    908.8914226...
+    908.648
     
     It may be more convinient to get properties with different units:
         
     >>> s1.get_property('rho', 'g/cm3')
-    0.90889142...
+    0.90864
     
     It is also possible to set some of the properties in different units:
         
@@ -374,7 +374,10 @@ class Stream:
         return self._price
     @price.setter
     def price(self, price):
-        self._price = float(price)
+        if np.isfinite(price):
+            self._price = float(price)
+        else:
+            raise AttributeError(f'price must be finite, not {price}')
     
     def isempty(self):
         """
@@ -439,18 +442,26 @@ class Stream:
         """[str] ID of chemical with the largest mol fraction in stream."""
         return self.chemicals.tuple[self.mol.argmax()].ID
 
-    def disconnect(self):
-        """Disconnect stream from unit operations."""
-        sink = self._sink
+    def disconnect_source(self):
+        """Disconnect stream from source."""
         source = self._source
-        if sink:
-            ins = sink.ins
-            index = ins.index(self)
-            ins[index] = None
         if source:
             outs = source.outs
             index = outs.index(self)
             outs[index] = None
+
+    def disconnect_sink(self):
+        """Disconnect stream from sink."""
+        sink = self._sink
+        if sink:
+            ins = sink.ins
+            index = ins.index(self)
+            ins[index] = None
+
+    def disconnect(self):
+        """Disconnect stream from unit operations."""
+        self.disconnect_source()
+        self.disconnect_sink()
     
     def _init_indexer(self, flow, phase, chemicals, chemical_flows):
         """Initialize molar flow rates."""
@@ -644,7 +655,7 @@ class Stream:
         >>> tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True) 
         >>> s1 = tmo.Stream('s1', Water=20, Ethanol=10, units='kg/hr')
         >>> s1.get_property('sigma', 'N/m') # Surface tension
-        0.063780393
+        0.06384
 
         """
         value = getattr(self, name)
@@ -1147,7 +1158,11 @@ class Stream:
         
         
         """
-        mol = self.mol
+        if self is s1: 
+            mol = self.mol.copy()
+        else:
+            mol = self.mol
+        
         s1.mol[:] = dummy = mol * split
         s2.mol[:] = mol - dummy
         
@@ -1820,7 +1835,7 @@ class Stream:
         >>> tmo.settings.set_thermo(['Water', 'Ethanol', 'Methanol'], cache=True) 
         >>> s1 = tmo.Stream('s1', Water=20, Ethanol=10, Methanol=10, units='m3/hr')
         >>> s1.get_concentration(('Water', 'Ethanol'))
-        array([27.671,  4.266])
+        array([27.672,  4.265])
 
         """
         F_vol = self.F_vol
