@@ -17,6 +17,7 @@
 """
 import numpy as np
 import flexsolve as flx
+from numba import njit
 
 __all__ = ('phase_fraction', 
            'solve_phase_fraction_Rashford_Rice',
@@ -25,7 +26,7 @@ __all__ = ('phase_fraction',
            'compute_phase_fraction_2N', 
            'compute_phase_fraction_3N')
 
-@flx.njitable(cache=True)
+@njit(cache=True)
 def as_valid_fraction(x):
     """Ensure that x is between 0 and 1."""
     if x < 0.:
@@ -34,6 +35,7 @@ def as_valid_fraction(x):
         x = 1.
     return x
 
+# @njit#(cache=True)
 def phase_fraction(zs, Ks, guess=None, za=0., zb=0.):
     """Return phase fraction for binary phase equilibrium."""
     N = zs.size
@@ -48,6 +50,7 @@ def phase_fraction(zs, Ks, guess=None, za=0., zb=0.):
                          'to find phase fraction')
     return as_valid_fraction(phase_fraction)
 
+# @njit#(cache=True)
 def solve_phase_fraction_iteration(zs, Ks, guess=0.5, za=0., zb=0.):
     """
     Return phase fraction for N-component binary phase equilibrium by 
@@ -67,7 +70,6 @@ def solve_phase_fraction_iteration(zs, Ks, guess=0.5, za=0., zb=0.):
     >>> import numpy as np
     >>> from thermosteam.equilibrium import (
     ...     solve_phase_fraction_iteration,
-    ...     phase_composition,
     ... )
     >>> F_air = 1
     >>> F_water = 1
@@ -96,26 +98,37 @@ def solve_phase_fraction_iteration(zs, Ks, guess=0.5, za=0., zb=0.):
     if y1 > y0 > 0.: return 0.
     if y0 < y1 < 0.: return 1.
     if y1 < y0 < 0.: return 0.
-    zs = zs[np.newaxis, :]
     if not 0. < guess < 1.: guess = 0.5
-    phi = np.ones([2, 1]); phi[:, 0] = [guess, 1. - guess]
+    phi = np.array([guess, 1. - guess])
     zc = np.ones([2, 1]); zc[:, 0] = [za, zb]
-    Ks = np.array([Ks, 1. / Ks])
+    N = zs.shape[0]
+    Ks_2d = np.ones([2, N])
+    zs_2d = np.ones([1, N])
+    zs_2d[0, :] = zs
+    Ks_2d[0, :] = Ks
+    Ks_2d[1, :] = 1. / Ks
     phi = flx.wegstein(f, phi, 1e-16, 
-                       args=(zs, Ks, zc), checkiter=False)
-    return phi[0, 0] / phi.sum()
+                       args=(zs_2d, Ks_2d, zc), checkiter=False)
+    return phi[0] / phi.sum()
 
-@flx.njitable(cache=True)
+@njit(cache=True)
 def phase_composition(zs, Ks, phi):
     return zs * Ks / (phi * Ks + (1. - phi))
 
-@flx.njitable(cache=True)
+@njit(cache=True)
 def compute_phase_fraction_iter(phi, zs, Ks, zc):
     ys = phase_composition(zs, Ks, phi)
-    phi = (ys * phi).sum(axis=1, keepdims=True)
-    phi += zc
-    return phi
-    
+    new_phi = np.zeros(phi.shape)
+    shape = Ks.shape
+    M = shape[0]
+    N = shape[1]
+    for i in range(M):
+        isum = 0.
+        for j in range(N): isum += ys[i, j]
+        new_phi[i] = isum * phi[i] + zc[i, 0]
+    return new_phi
+
+# @njit#(cache=True)
 def solve_phase_fraction_Rashford_Rice(zs, Ks, guess, za, zb):
     """
     Return phase fraction for N-component binary equilibrium by
@@ -141,7 +154,7 @@ def solve_phase_fraction_Rashford_Rice(zs, Ks, guess, za, zb):
                                 guess, 1e-16, 1e-16,
                                 args, checkiter=False)
 
-@flx.njitable(cache=True)
+@njit(cache=True)
 def phase_fraction_objective_function(phi, zs, K_minus_1, za, zb):
     """Phase fraction objective function."""
     numerator = - zs * K_minus_1
@@ -152,7 +165,7 @@ def phase_fraction_objective_function(phi, zs, K_minus_1, za, zb):
     phi = (numerator / denominator).sum()
     return phi - a + b
 
-@flx.njitable(cache=True)
+@njit(cache=True)
 def compute_phase_fraction_2N(zs, Ks):
     """Return phase fraction for 2-component binary equilibrium."""
     z1, z2 = zs
@@ -168,7 +181,7 @@ def compute_phase_fraction_2N(zs, Ks):
     K1z1_K2z2 = K1z1 + K2z2
     return (-K1z1_K2z2 + z1_z2)/(K1K2z1 + K1K2z2 - K1z2 - K1z1_K2z2 - K2z1 + z1_z2)
 
-@flx.njitable(cache=True)
+@njit(cache=True)
 def compute_phase_fraction_3N(zs, Ks):
     """Return phase fraction for 3-component binary equilibrium."""
     z1, z2, z3 = zs
