@@ -23,7 +23,9 @@ from ..indexer import ChemicalIndexer, MaterialIndexer
 from ..exceptions import InfeasibleRegion
 import numpy as np
 
-__all__ = ('Reaction', 'ParallelReaction', 'SeriesReaction')
+__all__ = ('Reaction', 'ReactionItem', 'ReactionSet', 
+           'ParallelReaction', 'SeriesReaction', 'ReactionSystem',
+           'Rxn', 'RxnS', 'RxnI', 'PRxn', 'SRxn', 'RxnSys')
 
 def get_stoichiometric_string(stoichiometry, phases, chemicals):
     if phases:
@@ -270,6 +272,19 @@ class Reaction:
             self._stoichiometry = np.zeros(chemicals.size)
             self._X_index = self._chemicals.index(reactant)
     
+    def __eq__(self, other):
+        try:
+            return all([
+                self._basis == other._basis,
+                self._phases == other._phases,
+                self._chemicals is other._chemicals,
+                self._X_index == other._X_index,
+                (self._stoichiometry == other._stoichiometry).all(),
+                self._X == other._X,
+            ])
+        except:
+            return False
+    
     def copy(self, basis=None):
         """Return copy of Reaction object."""
         copy = self.__new__(self.__class__)
@@ -383,7 +398,7 @@ class Reaction:
         product_index = self._chemicals.index(product)
         product_coefficient = self._stoichiometry[product_index]
         product_yield = product_coefficient * self.X
-        if self.basis != basis:
+        if basis and self.basis != basis:
             chemicals_tuple = self._chemicals.tuple
             reactant_index = self._X_index
             MW_reactant = chemicals_tuple[reactant_index].MW
@@ -778,13 +793,15 @@ class Reaction:
     def _rescale(self):
         """Scale stoichiometry to a per reactant basis."""
         new_scale = -self._stoichiometry[self._X_index]
+        if new_scale == 0.:
+            raise RuntimeError(f"reactant '{self.reactant}' does not participate in stoichiometric reaction")
         self._stoichiometry /= new_scale
     
     def __repr__(self):
         reaction = get_stoichiometric_string(self.stoichiometry, self.phases, self.chemicals)
         return f"{type(self).__name__}('{reaction}', reactant='{self.reactant}', X={self.X:.3g}, basis={repr(self.basis)})"
     
-    def show(self):
+    def _info(self):
         info = f"{type(self).__name__} (by {self.basis}):"
         rxn = get_stoichiometric_string(self.stoichiometry, self.phases, self.chemicals)
         if self.phases:
@@ -797,11 +814,14 @@ class Reaction:
         maxrxnlen = max([13, lrxn]) + 2
         maxcmplen = max([8, lcmp]) + 2
         X = self.X
-        info += "\n stoichiometry" + " "*(maxrxnlen - 13) + "reactant" + " "*(maxcmplen - 8) + '  X[%]'
+        info += "\nstoichiometry" + " "*(maxrxnlen - 13) + "reactant" + " "*(maxcmplen - 8) + '  X[%]'
         rxn_spaces = " "*(maxrxnlen - lrxn)
         cmp_spaces = " "*(maxcmplen - lcmp)
-        info += f"\n {rxn}{rxn_spaces}{cmp}{cmp_spaces}{X*100: >6.2f}"
-        print(info)
+        info += f"\n{rxn}{rxn_spaces}{cmp}{cmp_spaces}{X*100: >6.2f}"
+        return info
+    
+    def show(self):
+        print(self._info())
     _ipython_display_ = show
 
 
@@ -866,12 +886,8 @@ class ReactionSet:
     reactions : Iterable[Reaction]
     
     """
-    __slots__ = ('_basis',
-                 '_phases',
-                 '_stoichiometry', 
-                 '_X', '_X_index', 
-                 '_chemicals')
-
+    __slots__ = Reaction.__slots__
+    __eq__ = Reaction.__eq__
     copy = Reaction.copy
     phases = MaterialIndexer.phases
     _get_stoichiometry_by_mol = Reaction._get_stoichiometry_by_mol
@@ -949,20 +965,10 @@ class ReactionSet:
         """[2d array] Molecular weights of all chemicals."""
         return self._chemicals.MW[np.newaxis, :]
     
-    def _rescale(self):
-        """Scale stoichiometry to a per reactant basis."""
-        X_index = self._X_index
-        if self._phases:
-            index = (np.arange(len(X_index)), *zip(*X_index), np.newaxis)
-        else:
-            index = (np.arange(len(X_index)), X_index, np.newaxis)
-        new_scale = -self._stoichiometry[index]
-        self._stoichiometry /= new_scale
-    
     def __repr__(self):
-        return f"{type(self).__name__}([{', '.join([repr(i) for i in self])}])"
+        return f"{type(self).__name__}([{', '.join([repr(i).replace('ReactionItem', 'Reaction') for i in self])}])"
     
-    def show(self):
+    def _info(self, index_name='index'):
         info = f"{type(self).__name__} (by {self.basis}):"
         chemicals = self._chemicals
         phases = self._phases
@@ -974,16 +980,16 @@ class ReactionSet:
         maxcmplen = max([8, *[length(i) for i in cmps]]) + 2
         Xs = self.X
         N = len(Xs)
-        maxnumspace = max(length(string(N)) + 1, 5)
-        info += "\nindex" + " "*(max(2, maxnumspace-3)) + "stoichiometry" + " "*(maxrxnlen - 13) + "reactant" + " "*(maxcmplen - 8) + '  X[%]'
+        maxnumspace = max(length(string(N)) + 1, len(index_name))
+        info += f"\n{index_name}" + " "*(max(2, length(string(N)) + 1)) + "stoichiometry" + " "*(maxrxnlen - 13) + "reactant" + " "*(maxcmplen - 8) + '  X[%]'
         for N, rxn, cmp, X in zip(range(N), rxns, cmps, Xs):
             rxn_spaces = " "*(maxrxnlen - length(rxn))
             cmp_spaces = " "*(maxcmplen - length(cmp))
             num = string(N)
             numspace = (maxnumspace - length(num)) * " "
             info += f"\n[{N}]{numspace}{rxn}{rxn_spaces}{cmp}{cmp_spaces}{X*100: >6.2f}"
-        print(info)
-    _ipython_display_ = show
+        return info
+    _ipython_display_ = show = Reaction.show
         
 
 class ParallelReaction(ReactionSet):
@@ -1066,7 +1072,7 @@ class ParallelReaction(ReactionSet):
     
     Get net reaction conversion of reactants as a material indexer:
         
-    >>> mi = reaction.X_net
+    >>> mi = reaction.X_net(indexer=True)
     >>> mi.show()
     MaterialIndexer:
      (g) H2        0.5
@@ -1083,7 +1089,7 @@ class ParallelReaction(ReactionSet):
     ...    tmo.Reaction('H2 + O2 -> 2H2O',             reactant='H2',       X=0.7, **kwargs),
     ...    tmo.Reaction('Ethanol + O2 -> CO2 + 2H2O',  reactant='Ethanol',  X=0.1, **kwargs)
     ... ])
-    >>> ci = reaction.X_net
+    >>> ci = reaction.X_net(indexer=True)
     >>> ci.show()
     ChemicalIndexer:
      H2       0.7
@@ -1115,25 +1121,28 @@ class ParallelReaction(ReactionSet):
             rxn_dict[key] = rxn 
         return self.__class__(rxn_dict.values())
             
-    @property
-    def X_net(self):
-        """[ChemicalIndexer] Net reaction conversion of reactants."""
+    def X_net(self, indexer=False):
+        """Return net reaction conversion of reactants as a dictionary or
+        a ChemicalIndexer if indexer is True."""
         X_net = {}
         for i, j in zip(self.reactants, self.X):
             if i in X_net:
                 X_net[i] += j
             else:
                 X_net[i] = j
-        chemicals = self.chemicals
-        phases = self.phases
-        if phases:
-            phases = [i[0] for i in X_net]
-            mi = MaterialIndexer(phases=phases, chemicals=chemicals)
-            for i,j in X_net.items(): mi[i] = j
-            return mi                
+        if indexer:
+            chemicals = self.chemicals
+            phases = self.phases
+            if phases:
+                phases = [i[0] for i in X_net]
+                mi = MaterialIndexer(phases=phases, chemicals=chemicals)
+                for i,j in X_net.items(): mi[i] = j
+                return mi                
+            else:
+                data = chemicals.kwarray(X_net)
+                return ChemicalIndexer.from_data(data, NoPhase, chemicals, False)
         else:
-            data = chemicals.kwarray(X_net)
-            return ChemicalIndexer.from_data(data, NoPhase, chemicals, False)
+            return X_net
 
 
 class SeriesReaction(ReactionSet):
@@ -1157,22 +1166,225 @@ class SeriesReaction(ReactionSet):
         for i, j, k in zip(self._X_index, self.X, self._stoichiometry):
             material_array += material_array[i] * j * k
 
-    @property
-    def X_net(self):
-        """[ChemicalIndexer] Net reaction conversion of reactants."""
+    def X_net(self, indexer=False):
+        """Return net reaction conversion of reactants as a dictionary or
+        a ChemicalIndexer if indexer is True."""
         X_net = {}
         for i, j in zip(self.reactants, self.X):
             if i in X_net:
                 X_net[i] += (1 - X_net[i]) * j
             else:
                 X_net[i] = j
-        chemicals = self.chemicals
-        data = chemicals.kwarray(X_net)
-        return ChemicalIndexer.from_data(data, NoPhase, chemicals, False)
+        if indexer:
+            chemicals = self.chemicals
+            data = chemicals.kwarray(X_net)
+            return ChemicalIndexer.from_data(data, NoPhase, chemicals, False)
+        else:
+            return X_net
+
+class ReactionSystem:
+    """
+    Create a ReactionSystem object that can react a stream across a series of
+    reactions.
+    
+    Parameters
+    ----------
+    *reactions : Reaction, ParallelReaction, or SeriesReaction
+        All reactions within the reaction system.
+    
+    Examples
+    --------
+    Create a reaction system for cellulosic fermentation of biomass:
+    
+    >>> from thermosteam import Rxn, RxnSys, PRxn, SRxn, settings, Chemical, Stream
+    >>> cal2joule = 4.184
+    >>> Glucan = Chemical('Glucan', search_db=False, formula='C6H10O5', Hf=-233200*cal2joule, phase='s', default=True)
+    >>> Glucose = Chemical('Glucose', phase='s')
+    >>> CO2 = Chemical('CO2', phase='g')
+    >>> HMF = Chemical('HMF', search_ID='Hydroxymethylfurfural', phase='l', default=True)
+    >>> Biomass = Glucose.copy(ID='Biomass')
+    >>> settings.set_thermo(['Water', 'Ethanol', 'LacticAcid', HMF, Glucose, Glucan, CO2, Biomass])
+    
+    >>> saccharification = PRxn([
+    ...     Rxn('Glucan + H2O -> Glucose', reactant='Glucan', X=0.9),
+    ...     Rxn('Glucan -> HMF + 2H2O', reactant='Glucan', X=0.025)
+    ... ])
+    >>> fermentation = SRxn([
+    ...     Rxn('Glucose -> 2LacticAcid', reactant='Glucose', X=0.03),
+    ...     Rxn('Glucose -> 2Ethanol + 2CO2', reactant='Glucose', X=0.95),
+    ... ])
+    >>> cell_growth = Rxn('Glucose -> Biomass', reactant='Glucose', X=1.0)
+    >>> cellulosic_rxnsys = RxnSys(saccharification, fermentation, cell_growth)
+    >>> cellulosic_rxnsys.show()
+    ReactionSystem:
+    index  reaction
+    [0]    ParallelReaction (by mol):
+           subindex  stoichiometry              reactant    X[%]
+           [0]       Water + Glucan -> Glucose  Glucan     90.00
+           [1]       Glucan -> 2 Water + HMF    Glucan      2.50
+    [1]    SeriesReaction (by mol):
+           subindex  stoichiometry                 reactant    X[%]
+           [0]       Glucose -> 2 LacticAcid       Glucose     3.00
+           [1]       Glucose -> 2 Ethanol + 2 CO2  Glucose    95.00
+    [2]    Reaction (by mol):
+           stoichiometry       reactant    X[%]
+           Glucose -> Biomass  Glucose   100.00
+    
+    Compute the flux of glucan through saccharification reactions:
+    
+    >>> feed = Stream('feed', Glucan=1.0, Water=5.0)
+    >>> cellulosic_rxnsys.reactant_flux(feed, index=0)
+    0.925
+    
+    Compute the flux of glucan through glucan to glucose saccharification:
+    
+    >>> cellulosic_rxnsys.reactant_flux(feed, index=0, subindex=0)
+    0.9
+    
+    Compute the flux of glucose through cell growth:
+        
+    >>> cellulosic_rxnsys.reactant_flux(feed, index=2)
+    0.04365
+    
+    Compute the flux of glucose through ethanol production:
+    
+    >>> cellulosic_rxnsys.reactant_flux(feed, index=1, subindex=1)
+    0.8293
+    
+    Notice how reacting the stream leads to ethanol production equivalent
+    of 2x the glucose flux to that reaction:
+    
+    >>> cellulosic_rxnsys(feed)
+    >>> feed.show()
+    Stream: feed
+     phase: 'l', T: 298.15 K, P: 101325 Pa
+     flow (kmol/hr): Water       4.15
+                     Ethanol     1.66
+                     LacticAcid  0.054
+                     HMF         0.025
+                     Glucan      0.075
+                     CO2         1.66
+                     Biomass     0.0437
+    
+    """
+    __slots__ = ('_reactions',
+                 '_basis',
+                 '_chemicals',
+                 '_phases')
+    
+    def __init__(self, *reactions, basis=None):
+        if not reactions: raise ValueError('Reactions cannot be empty')
+        self._reactions = reactions
+        try: self._phases, = set([i._phases for i in reactions])
+        except: raise ValueError('all reactions must have the same phases')
+        try: self._chemicals, = set([i._chemicals for i in reactions])
+        except: raise ValueError('all reactions must have the same chemicals')
+        try: self._basis, = set([i._basis for i in reactions])
+        except: raise ValueError('all reactions must have the same basis')
+        
+    force_reaction = Reaction.force_reaction
+    adiabatic_reaction = Reaction.adiabatic_reaction
+    __call__ = Reaction.__call__
+    show = Reaction.show
+    
+    @property
+    def reactions(self):
+        return self._reactions
+    
+    def _reaction(self, material):
+        basis = self._basis
+        for i in self._reactions: 
+            if i._basis != basis: raise RuntimeError('not all reactions have the same basis')
+            i._reaction(material)
+        
+    def X_net(self, indexer=False):
+        """Return net reaction conversion of reactants as a dictionary or
+        a ChemicalIndexer if indexer is True."""
+        X_net = {}
+        for rxn in self._reactions:
+            for i, j in rxn.X_net().items():
+                if i in X_net:
+                    X_net[i] += (1 - X_net[i]) * j
+                else:
+                    X_net[i] = j
+        if indexer:
+            chemicals = self._chemicals
+            data = chemicals.kwarray(X_net)
+            return ChemicalIndexer.from_data(data, NoPhase, chemicals, False)
+        else:
+            return X_net
+        
+    def reactant_flux(self, material, index, subindex=None):
+        """
+        Return the amount of reactant being reacted in a reaction.
+        
+        Parameters
+        ----------
+        material : Stream or array
+            Material entering reaction system.
+        index : int
+            Index of reaction to calculate reactant flux.
+        subindex : int, optional
+            Subindex of reaction to calculate reactant flux.    
+        
+        """
+        material_array = as_material_array(material,
+                                           self._basis,
+                                           self._phases,
+                                           self._chemicals)
+        values = get_array_values(material_array)
+        preconverted_material = values.copy()
+        reactions = self.reactions
+        for i, rxn in enumerate(reactions):
+            if i == index: break
+            rxn(preconverted_material)
+        reaction = reactions[index]
+        if subindex is not None:
+            if isinstance(reaction, SeriesReaction):
+                reactions = reaction
+                for i, rxn in enumerate(reactions):
+                    if i == subindex: break
+                    rxn(preconverted_material)
+            reaction = reaction[subindex]
+            X_index = reaction._X_index
+            return reaction.X * preconverted_material[X_index]
+        elif isinstance(reaction, SeriesReaction):
+            raise ValueError('must pass subindex if the index refers to a SeriesReaction object')
+        elif isinstance(reaction, ParallelReaction):
+            X_index = reaction._X_index
+            return (reaction.X * preconverted_material[X_index]).sum()
+        else:
+            X_index = reaction._X_index
+            return reaction.X * preconverted_material[X_index]
+        
+        
+    def __repr__(self):
+        return f"{type(self).__name__}({', '.join([repr(i) for i in self.reactions])})"
+    
+    def _info(self):
+        indexed_info = f"{type(self).__name__}:\n"
+        isa = isinstance
+        infos = [(i._info('subindex') if isa(i, ReactionSet) else i._info()) 
+                 for i in self._reactions]
+        N = len(infos)
+        index_name = 'index'
+        maxnumspace = max(len(str(N)) + 1, len(index_name))
+        header = f"{index_name}" + " "*(max(2, maxnumspace-3))
+        indexed_info += header + "reaction"
+        info_dim = '\n' + len(header) * " "
+        for index, info in enumerate(infos):
+            num = str(index) 
+            numspace = (maxnumspace - len(num)) * " "
+            info = info.replace('\n', info_dim)
+            indexed_info += f"\n[{num}]{numspace}{info}"
+        return indexed_info    
+        
+    _ipython_display_ = show = Reaction.show
 
 # Short-hand conventions
-# Rxn = Reaction
-# RxnI = ReactionItem
-# RxnS = ReactionSet
-# PRxn = ParallelReaction
-# SRxn = SeriesReaction
+Rxn = Reaction
+RxnI = ReactionItem
+RxnS = ReactionSet
+PRxn = ParallelReaction
+SRxn = SeriesReaction
+RxnSys = ReactionSystem
\ No newline at end of file
